#include "giacPCH.h"
#include <stdlib.h>
#include <string.h>
#include <libfx.h>

#include "menu_config.h"
#include "console.h"

extern giac::context * contextptr; // in Taumath.cpp

#define WIDTH 128
#define HEIGHT 64

#define menu_free_item(item) if(item) free(item->str)

int test;

void draw_menu(Menu7* menu)
{
	int x, y, i; 
	unsigned int key;
	int longest = 0, tmp;
	int border = 1, margin = 2, check_box_width = 15;
	int	item_height = 7, letter_width = 4;
	int cursor_y = 0;
	DISPBOX box;

	for(i=0; i < menu->items_number; i++) {
		tmp = strlen(menu->items[i]->str) * letter_width;
		if(menu->items[i]->type == CHECK_BOX) tmp += check_box_width;

		if(tmp > longest) longest = tmp;
	}

	box.left = (WIDTH/2) - (longest/2) - margin - border;
	box.right = box.left + longest + 2*margin + 2*border;
	box.bottom = (HEIGHT/2) + (menu->items_number * item_height)/2 + border;
	box.top = HEIGHT - box.bottom;

	Bdisp_AreaClr_VRAM(&box);

	Bdisp_AreaReverseVRAM(box.left, box.top, box.right, box.bottom);

	Bdisp_AreaReverseVRAM(box.left + border, box.top + border,
							box.right - border, box.bottom - border);

	while (key != KEY_CTRL_EXIT) {
		for(i=0; i < menu->items_number; i++) {
			PrintMini(box.left + border + margin, box.top + margin + item_height*i, (unsigned char*)menu->items[i]->str, MINI_OVER);
			if(menu->items[i]->type == CHECK_BOX) {
				if((menu->items[i]->action.val.value)) {
					if(*(menu->items[i]->action.val.value)) PrintMini(box.right - border - margin - (check_box_width/2) , box.top + margin + item_height*i, (unsigned char*)"[x]", MINI_OVER);
					else PrintMini(box.right - border - margin - (check_box_width/2) , box.top + margin + item_height*i, (unsigned char*)"[ ]", MINI_OVER);
				}
			}
		}
		PrintMini(box.left + border + margin, box.top + margin + item_height*cursor_y, (unsigned char*)menu->items[cursor_y]->str, MINI_REV);
		if(menu->items[cursor_y]->type == CHECK_BOX) {
			if((menu->items[cursor_y]->action.val.value)) {
				if(*(menu->items[cursor_y]->action.val.value)) PrintMini(box.right - border - margin - (check_box_width/2) , box.top + margin + item_height*cursor_y, (unsigned char*)"[x]", MINI_OVER);
				else PrintMini(box.right - border - margin - (check_box_width/2) , box.top + margin + item_height*cursor_y, (unsigned char*)"[ ]", MINI_OVER);
			}
		}
		GetKey(&key);
		
		if (key == KEY_CTRL_DOWN && cursor_y < menu->items_number-1) cursor_y++;	
		if (key == KEY_CTRL_UP && cursor_y > 0) cursor_y--;

		if (key == KEY_CTRL_EXE) {
			//menu_test();
			if(menu->items[cursor_y]->type == CHECK_BOX) {
				if(menu->items[cursor_y]->action.val.value != NULL) { //Changing the state of the variable, as a boolean
					if(*(menu->items[cursor_y]->action.val.value)) *(menu->items[cursor_y]->action.val.value) = 0;
					else *(menu->items[cursor_y]->action.val.value) = 1;

					if (menu->items[cursor_y]->action.val.save_function != NULL) (*menu->items[cursor_y]->action.val.save_function)();
				}
			}
			else if(menu->items[cursor_y]->type == FUNCTION_CALL) {
				if(menu->items[cursor_y]->action.function != NULL) {
					(*(menu->items[cursor_y]->action.function))(); //Calling the function
					break;
				}
			}
		}
	}
}

int * get_python_address(){
  return &giac::python_compat(contextptr);
}

int * get_angle_address(){
  return &giac::angle_mode(contextptr);
}

void menu_setup()
{
  int number = 7;
  Menu7 menu;
  menu.items = (Menu7_Item **) malloc(sizeof(Menu7_Item*) * number);
  
  menu.items[0] = menu_create_item("Help Console", FUNCTION_CALL, menu_help_console, (void*)NULL);
  menu.items[1] = menu_create_item("Help 2d editor", FUNCTION_CALL, menu_help_eqw, (void*)NULL);
  menu.items[2] = menu_create_item("Help Graph", FUNCTION_CALL, menu_help_graph, (void*)NULL);
  menu.items[3] = menu_create_item("Help Script edit", FUNCTION_CALL, menu_help_script, (void*)NULL);
  // menu.items[0] = menu_create_item("Clear console", FUNCTION_CALL, menu_clear, (void*)NULL);
  menu.items[4] = menu_create_item("Python", CHECK_BOX, get_python_address(), save_config);
  menu.items[5] = menu_create_item("Degrees", CHECK_BOX, get_angle_address(), save_config);
  menu.items[6] = menu_create_item("About", FUNCTION_CALL, menu_about, (void*)NULL);
  if(menu.items[0] && menu.items[1] && menu.items[2] && menu.items[3] && menu.items[4] && menu.items[5] && menu.items[6])
    menu.items_number = number;
  else {
    for (int i=0;i<number;++i)
      menu_free_item(menu.items[i]);
  }
  
  draw_menu(&menu);
  
  menu_free_item(menu.items[0]);
  //menu_free_item(menu.items[1]);
  free(menu.items);
}

void menu_clear(){
  Console_Init();
  Console_Clear_EditLine();
  Console_Disp();
}

void menu_about()
{
	unsigned int key;
	PopUpWin(6);
	PrintMini(12, 6, (unsigned char*)"Short help: shift-SETUP", MINI_OVER);
	PrintMini(12, 14, (unsigned char*)"KhiCAS, B. Parisse et al", MINI_OVER);
	PrintMini(20, 22, (unsigned char*)"Universite de Grenoble", MINI_OVER);
	PrintMini(12, 38, (unsigned char*)"Interface by Mike, Nemh,", MINI_OVER);
	PrintMini(20, 46, (unsigned char*)"G.Maia, LePhenixNoir...", MINI_OVER);
	PrintMini(12, 54, (unsigned char*)"GPL2 license, v. 0.5", MINI_OVER);
	GetKey(&key);
	return;
}

void menu_help_eqw()
{
	unsigned int key;
	PopUpWin(6);
	PrintMini(12, 6, (unsigned char*)"2d editor (FRAC) shortcuts", MINI_OVER);
	PrintMini(12, 14, (unsigned char*)"F1: algebra", MINI_OVER);
	PrintMini(12, 22, (unsigned char*)"F2: calculus, analysis", MINI_OVER);
	PrintMini(12, 30, (unsigned char*)"F3: Edit, fontsize 8 or 6", MINI_OVER);
	PrintMini(12, 38, (unsigned char*)"F4: all commands", MINI_OVER);
	PrintMini(12, 46, (unsigned char*)"F5: alpha lock", MINI_OVER);
	PrintMini(12, 54, (unsigned char*)"F6: eval evalf regroup", MINI_OVER);
	GetKey(&key);
	return;
}

void menu_help_script()
{
	unsigned int key;
	PopUpWin(6);
	PrintMini(12, 6, (unsigned char*)"Script editor shortcuts", MINI_OVER);
	PrintMini(12, 14, (unsigned char*)"shift SETUP change font", MINI_OVER);
	PrintMini(12, 22, (unsigned char*)"CLIP: begin end select", MINI_OVER);
	PrintMini(12, 30, (unsigned char*)"AC/ON clear current line", MINI_OVER);
	PrintMini(12, 38, (unsigned char*)"EXE test", MINI_OVER);
	PrintMini(12, 46, (unsigned char*)"F5: alpha lock", MINI_OVER);
	PrintMini(12, 54, (unsigned char*)"F6: file save/insert", MINI_OVER);
	GetKey(&key);
	return;
}

void menu_help_graph()
{
	unsigned int key;
	PopUpWin(6);
	PrintMini(12, 6, (unsigned char*)"Graph viewer shortcuts", MINI_OVER);
	PrintMini(12, 14, (unsigned char*)"output of plot commands", MINI_OVER);
	PrintMini(12, 22, (unsigned char*)"zoom: + in, - out", MINI_OVER);
	PrintMini(12, 30, (unsigned char*)"yout: (-), autoscale: *", MINI_OVER);
	PrintMini(12, 38, (unsigned char*)"orthonormalize: /", MINI_OVER);
	PrintMini(12, 46, (unsigned char*)"move axes: keypad", MINI_OVER);
	PrintMini(12, 54, (unsigned char*)"axes on/off: VARS", MINI_OVER);
	GetKey(&key);
	return;
}

void menu_help_console()
{
	unsigned int key;
	PopUpWin(6);
	PrintMini(12, 6, (unsigned char*)"F3 key: view/edit", MINI_OVER);
	PrintMini(12, 14, (unsigned char*)"EXE in history: copy", MINI_OVER);
	PrintMini(12, 22, (unsigned char*)"List, Matr: menus", MINI_OVER);
	PrintMini(12, 30, (unsigned char*)"Help: from catalog", MINI_OVER);
	PrintMini(20, 38, (unsigned char*)"select cmd, press F6", MINI_OVER);
	PrintMini(12, 46, (unsigned char*)"F4 Enter catalog", MINI_OVER);
	PrintMini(12, 54, (unsigned char*)"AC on empty line: clear", MINI_OVER);
	GetKey(&key);
	return;
}

Menu7_Item* menu_create_item(const char* str, Menu7_Item_Type type, void* other, void* save_func)
{
	Menu7_Item* item = malloc(sizeof(Menu7_Item));
	if(item == NULL) return NULL;

	item->str = malloc(sizeof(char)*(strlen(str) + 1));
	if(item->str == NULL) {
		free(item);
		return NULL;
	}
	else strcpy(item->str, str);

	item->type = type;

	if(type == CHECK_BOX) {
		item->action.val.value = other;
		item->action.val.save_function = save_func;
	}
	else if(type == FUNCTION_CALL) item->action.function = other;

	return item;
}

void save_config()
{
#ifdef TEX
  memory_save(CONFIG_FILE, (int*)get_tex_flag_address(), 4); // 4==sizeof(int)
#endif
}

void load_config()
{
#ifdef TEX
  if(memory_exists(CONFIG_FILE)) 
    *(int*)(get_tex_flag_address()) = *((int*)memory_load(CONFIG_FILE));
#endif
}
